import cors from 'cors';
import dateformat from 'dateformat';
import Debug from 'debug';
import express from 'express';
import cacheController from 'express-cache-controller';
import fs from 'fs';
import jsonfile from 'jsonfile';
import path from 'path';

const debugRequest = Debug('api:sermon:Request');
const debugCors = Debug('api:sermon:Cors');
const debugData = Debug('api:sermon:Data');

const regex = RegExp('\.hillcrestpresbyterian\.org$');
const sermonsPath = path.join(__dirname, '../sermons.json');

const corsOptions = {
  origin: (origin: any, callback: any) => {
    if (!origin
        || regex.test(origin)
        || (process.env.NODE_ENV && process.env.NODE_ENV.toUpperCase() === 'DEVELOPMENT')) {
      debugCors('origin allowed: ' + origin);
      debugCors('is cors match: ' + regex.test(origin));

      callback(null, true);
    } else {
      debugCors('origin denied: ' + origin);
      callback(new Error('Not allowed by CORS'));
    }
  },
};

let sermonData: any;
let lastModifiedHeader: string;
let lastModified: Date;

function loadSermonData() {
  // TODO: Last-Modified should not be hardcoded but pulled from DB.
  // 'Mon, 12 Aug 2019 00:23:00 GMT''ddd, dd mmm yyyy HH:MM:SS GMT'
  const stat = fs.statSync(sermonsPath);

  if (!lastModified || stat.mtime.valueOf() !== lastModified.valueOf()) {
    lastModified = stat.mtime;
    lastModifiedHeader = dateformat(lastModified, 'UTC:ddd, dd mmm yyyy HH:MM:ss') + ' GMT';
    debugData('lastModified: ' + lastModifiedHeader);
    sermonData = jsonfile.readFileSync(sermonsPath);
  } else {
    debugData('Skipping sermon.json load');
  }
}

debugData('sermons.json: ' + sermonsPath);
debugData('Initial sermon load');
loadSermonData();

const sermons = express.Router();

fs.watch(sermonsPath, {
  encoding: 'utf8',
  persistent: false,
}, (event, filename) => {
  debugData('watch event \'' + event + '\': ' + filename);
  if (event === 'change' && filename === 'sermons.json') {
    debugData('Attempting reload sermon data');
    loadSermonData();
  }
});

sermons.get('/sermons', cors(corsOptions), (req, res) => {

  debugRequest(req.query);

  res.cacheControl = {
      maxAge: 600,
   };

  res.setHeader('Last-Modified', lastModifiedHeader);
  // if a callback parameter is present the response is jsonp
  // if a callback parameter is not present then response is json
  res.jsonp(sermonData);
});

export = sermons;
